body, html {
	height: 100%;
	margin: 0;
	overflow:hidden;
	font-family: helvetica;
	font-weight: 100;
}

#layout {
	z-index:0;
	width: 100%; 
	height: 100%;
	top: -32px;
}

#logcontainer {
	z-index:2;
	width: 75%;
	height: 5%;
	display: block;
	border: 1px solid gray;
	text-align: center;
	overflow: hidden;
	left: 12.5%;
	resize: vertical; 
	-moz-resize: vertical; 
	border: solid 3px  #2290c9; 
	border-radius: 10px; 
	padding: 0.5%; position: absolute; 
	bottom: 10px; 
	-webkit-transform: rotate(-180deg); 
	-moz-transform: rotate(-180deg); 
	-o-transform: rotate(-180deg); 
	-ms-transform: rotate(-180deg); 
	-transform-transform: rotate(-180deg);
	background: rgba(255, 255, 255, 0.9);
	visibility: hidden;
	max-height: 98%;
}

#log {
	/*position: absolute;*/
	z-index:2;
	width: 99%;
	height: 100%;
	text-align:left;
	left: 0.5%;
	margin: 0px 0px 0px 0px;
	overflow-y: auto;
	overflow-x: auto;
	white-space: nowrap;
	line-height: 85%;
	resize: vertical; 
	-moz-resize: vertical; 
	-webkit-transform: rotate(-180deg); 
	-moz-transform: rotate(-180deg); 
	-o-transform: rotate(-180deg); 
	-ms-transform: rotate(-180deg); 
	-transform-transform: rotate(-180deg);

}

#layout, #layout1,#layout2,#layout3,#layout4,#layout5, #download{
	width: 100%;
	height: 100%;
}

#chartlayoutBar,#chartlayoutScatter, #chartlayoutLine, #chartlayoutStack, #chartlayoutPie {
	float: left;
	width: 33%;
	height: 50%;
	display: visible;
}

#sequence {
  	position: absolute;
	top: 10px;
	left: 10px;
  	width: 600px;
  	height: 70px;
}

#legend {
  	padding: 10px 0 0 3px;
}

#sequence text, #legend text {
  	font-weight: 600;
  	fill: #fff;
}

#sidebarLegend {
	z-index: 2;
  	visibility: hidden;
	position: absolute;
}

#explanation {
	  position: absolute;
	  width: 140px;
	  text-align: center;
	  color: #666;
	  z-index: -1;
}

#percentage {
  	font-size: 2.5em;
}

#load {
	position: absolute;
	z-index: 1000;
	height: 100%;
	width: 100%;
	top: 0px;
	left: 0px;
	display: none;
}

#loadBackground {
	opacity: 0.75;
	background: grey;
	z-index: 1000;
	height: 100%;
	width: 100%;
	top: 0px;
	left: 0px;
}

.ajax-loader {
	position: absolute;
	left: 50%;
	top: 50%;
	margin-left: -32px; /* -1 * image width / 2 */
	margin-top: -32px; /* -1 * image height / 2 */
	height: 5%;
	width: 2.5%;
}

#loading {
	position: absolute;
	left: 50%;
	top: 50%;
	margin-left: -45px; /* -1 * image width / 2 */
	margin-top: 25px; /* -1 * image height / 2 */
	height: 5%;
	width: 2.5%;
	color: black;
	font-weight: bold;
	font: 22px/1.5 "Helvetica Neue", Arial, Helvetica, sans-serif;
}
