.leaflet-center {
	width: 40%;
	margin-left: auto;
	margin-right: auto;
	position: relative;
}
.leaflet-control-geosearch, .leaflet-control-geosearch ul {
	border-radius: 7px;
	background: none repeat scroll 0 0 rgba(0, 0, 0, 0.25);
	margin: 10px 0 0 0;
	padding: 5px;
	width: 100%;
	height: auto;
}
.leaflet-control-geosearch-msg ul {
	list-style: none;
	display: none;
	height: auto;
	background: none;
	padding: 0;
}
.leaflet-control-geosearch ul li {
	background: none repeat scroll 0 0 rgba(255, 255, 255, 0.75);
    border-radius: 4px;
    margin: 2px 0;
    padding: 4px;
    font: 12px arial;
    text-indent: 4px;
}
.leaflet-container .leaflet-control-geosearch input {
	width: 100%;
	height: 28px;
	padding: 0;
	text-indent: 8px;
	background: rgba(255, 255, 255, 0.75);			
	border-radius: 4px;
	border: none;
}