circle.node {
  fill: #fff;
  stroke: steelblue;
  stroke-width: 1.5px;
}

line.dline{
	stroke: black;
	stroke-width: 0.5;
	stroke-dasharray: 2.5,5;
	stroke-opacity: 0.5;
}

.track circle {
  fill: none;
  stroke: #aaa;
  stroke-width: 1px;
}

circle.selectednode {
  fill: steelblue;
  stroke: #335E82;
  stroke-width: 1.5px;
}

.node, text {
  font: 10px sans-serif;
}

.selectednode {
  font: bold 12px sans-serif;
  background-color: #ff0000;
}


.link {
  fill: none;
  stroke: #ccc;
  stroke-width: 1.5px;
}

text {
  fill: #000;
}
